@@ -1,15 +1,10 @@
-from viset.caltech import Caltech256
+from viset.library.caltech import Caltech256
 from viset.dataset import Viset
 from viset.show import imshow
 
 if __name__ == '__main__':
   dbfile = Caltech256().export()
-  db = Viset(dbfile, async=False, datastep=100)
-  for (im,annotation) in db[0]:
+  db = Viset(dbfile, verbose=True)
+  for (im,annotation) in db.annotation.categorization:
     print 'Image=' + str(annotation['id_img']) + ' Category=' + annotation['category']
     imshow(im)
-
-  # Cleanup cached data (optional)
-  db.delete()
-
-  
