@@ -3,7 +3,7 @@ from viset.dataset import Viset
 from viset.show import imshow
 
 def main():
-    dbfile = WeizmannHorsesSingleScale().export()
+    dbfile = WeizmannHorsesSingleScale(verbose=True).export()
     db = Viset(dbfile, verbose=True)
     for (im,annotation) in db.annotation.segmentation:
         imshow(im, title='horses')
