@@ -3,12 +3,14 @@
 import os
 from os import path
 import hashlib
+import numpy
 import urlparse
 from viset.util import isarchive, isurl, isimg, ishdf5, isfile
 import viset.download
 import pylab
 import string
 import shutil
+import h5py
 
 class Cache():
     _cacheroot = os.environ.get('VISYM_CACHE')
@@ -150,9 +152,24 @@ class Cache():
         
         quietprint('[viset.cache][HIT]: loading \'' + filename + '\'', self._verbose)                                         
         return self.load(filename, reader=urlopts['reader'], id=urlopts['id'])
-    
+
+    def put(self, obj):
+        """Put a numpy object into cache and return cached file""" 
+        if 'numpy' not in str(type(obj)):
+            if type(obj) in [list, tuple]:
+                obj = numpy.array(obj)  # type coersion
+            else:
+                raise ValueError('numpy object required for caching')                    
+        byteview = obj.view(numpy.uint8)
+        hash = hashlib.sha1(byteview).hexdigest() # byte view sha1
+        filename = self.abspath(hash + '.h5')
+        f = h5py.File(filename, 'w')
+        f['obj'] = obj
+        f.close()
+        return filename
+        
     def get(self, url, sha1=None):
-        """Get an element from cache""" 
+        """Get an object from cache""" 
         try:
             return self.fetch(url, sha1=sha1)
         except KeyboardInterrupt:
@@ -168,14 +185,14 @@ class Cache():
                 raise
             
     def discard(self, url):
-        """Remove element from cache"""
+        """Delete element from cache"""
         if path.isfile(self.cachefile(url)):
             os.remove(self.cachefile(url))
         if path.isdir(self.cachedir(url)):
             shutil.rmtree(self.cachedir(url))
         
     def size(self, source=_cacheroot):
-        """Recursively compute the size of a directory: http://snipplr.com/view/47686/"""
+        """Recursively compute the size of a cache directory: http://snipplr.com/view/47686/"""
         total_size = os.path.getsize(source)
         for item in os.listdir(source):
             itempath = os.path.join(source, item)
@@ -279,69 +296,3 @@ class CachedImage(CachedResult):
         return str('<viset.image: obj=' + str(type(self._obj)) + ', cached=' + str(self._cache.iscached(self._url)) + ', URL=\'' + str(self._url) + '\'>')        
 
     
-##     convert from viset:// to file:// or http://
-##     viset:path_to_resource
-##     viset://domain.com/
-
-##     cannot access http://path.to.h5?query since a row may contain a viset.cache which cannot be handled 
-##     viset://dbname.h5?view=''&group=''&table=''&row=''&field=''
-##     viset://dbname.h5?path='/root/annotation/categorization/0/url'
-##     URI: viset.cache:/path/to/resource => URL FILE://{VISET_CACHE}/path/to/resource
-
-##     viset.cache.mnist:path/to/mnistbinary.gz?id=0
-
-##     can still include relative uri in the viset.  relative uris are relative to base uri FILE://{VISET_CACHE}/path
-
-##     viset.http://domain.com/path.h5
-##     viset.hdfs:path.h5?q='query'
-
-##     viset:/absolute/path.h5?im=0   
-##     viset:path.h5?im=0    # relative url, get in cache
-##     viset://domain.com/absolute/path.h5?view='image'&id=0        # query server for just row
-##     viset.http://domain.com/absolute/path.h5?view='image'&id=0    # download entire viset via http
-##     viset.hdfs://domain.com/absolute/path.h5?view='categorization'&id=0        # access viset via hdfs
-
-##     viset.mnist:/path.gz#frag  pass to viset mnist handler for reading 
-    
-##     do we just cache function calls to viset.types.Image object?  no, want to make sure that separate calls to Image from same package use cached data
-##     use queries and fragments
-##     querystrings are passed to the server to return a document based on the query.  this query file is returned from the cache
-##     fragments are handled by the cache itself.  
-##     who unzips? who loads?  file cache vs. object cache?
-    
-##     fragments are passed to
-    
-##     image and annotations are handled by viset uris.  don't bother importing on the client and pickling, send viset uri and require the client to resolve to locations and unpickle
-    
-##     foo://username:password@example.com:8042/over/there/index.dtb?type=animal&name=narwhal#nose
-
-##     if we ahve a local copy of a viset, how do we share with clients?  need to copy it somehow, or pass a reference.
-## how can we still pass rows?
-## must put the file in a place accessible by clients
-## maybe expose viset://master.ip/path.h5
-## no, pass package dependencies along with the rows so we download only those packages necessary
-## packages must be accessible via http (by construction of viset)
-
-## just pass the package url to the clients
-## viset.http://packageurl.tgz#fragment
-## viset.dbname: relative url
-
-## do not store separate packages, store urls with fragments.  the caching takes care of downloading and unpacking once we use it
-
-## viset://package.tgz?reader=mnist&id=0   cache hashes url (without querystring) to filename for reuse between visets 
-## viset defaults to http, but need viset scheme to handle querystring appropriately .  use viset.hdfs for nested schemes (inner fetches file, outer handles query and fragment)
-## client does not know or care what database this came from.  it downloads and caches for everyone unless overridden for private cache
-
-## caching on shared machine?  winner locks the file and os cannot open it for writing.  wait and retest if cached
-
-## https://github.com/smontanaro/pylockfile
-## http://docs.python.org/2/library/fcntl.html#fcntl.lockf
-## http://www.evanfosmark.com/2009/01/cross-platform-file-locking-support-in-python/
-## http://amix.dk/blog/post/19531
-
-## viset image object has a standard viset: scheme for caching and importing and image
-
-## for shared resources, can pass URL FILE://
-## never see any relative urls stored in viset everything is a URL and caching is transparent
-
-
