@@ -30,7 +30,7 @@ class Viset(object):
     _async = None
     _createviews = [ImageView()]    
     
-    def __init__(self, dburl=None, task=None, strategy=None, kfold=0, datastep=1, async=True, verbose=False):
+    def __init__(self, dburl=None, task=None, strategy=None, kfold=0, datastep=1, async=False, verbose=False):
         self._verbose = verbose            
         self._cache._verbose = verbose
         self._dburl = dburl
@@ -42,25 +42,13 @@ class Viset(object):
             pass
         
     def __getattr__(self, viewname):
-        """Return attributes"""
-
-        # Static attributes
-        if viewname in ['_cache','cache']:
-            return self._cache
-        elif viewname in ['_obj','obj']:
-            return self._obj
-        elif viewname in ['_dburl','url','_url']:
-            return self._dburl
-        elif viewname in ['_verbose']:
-            return self._verbose
-        else:
-            # Dynamic attributes
-            for node in self._obj.root:
-                if node._v_attrs.viewname == viewname:
-                    viewmodule = sys.modules['viset.views']
-                    viewclass = getattr(viewmodule, node._v_attrs.viewclass)
-                    return viewclass(node, self._obj.root, verbose=self._verbose)
-            raise IOError('undefined attribute ' + viewname)
+        """Return static and dynamic attributes"""
+        for node in self._obj.root:
+            if node._v_attrs.viewname == viewname:
+                viewmodule = sys.modules['viset.views']
+                viewclass = getattr(viewmodule, node._v_attrs.viewclass)
+                return viewclass(node, self)
+        raise AttributeError, viewname
         
     def __del__(self):
         if self._obj is not None:
@@ -73,7 +61,7 @@ class Viset(object):
         self._obj.flush()
 
     def summary(self):
-        print 'Database URL: ' + self.url
+        print 'Database URL: ' + self._dburl
         print 'Database name: ' + str(self.name())            
         print 'Database views: ' + str(self.views())
         print 'Number of images: ' + str(len(self.image))
@@ -82,7 +70,7 @@ class Viset(object):
         print 'HDF5 file structure: ' 
         print '  ' + "\n  ".join(str(self._obj).split("\n"))
         print 'Cache directory: ' + self._cache._cacheroot
-        print 'Database local file: ' + self._cache.get(self.url)
+        print 'Database local file: ' + self._cache.get(self._dburl)
         return str()
         
     def open(self, dburl):
@@ -158,7 +146,7 @@ class Viset(object):
     def views(self):
         """List of views in database"""    
         viewlist = []
-        for node in self.obj.root._f_walknodes():
+        for node in self._obj.root._f_walknodes():
             viewlist.append(str(node._v_pathname).replace('/','.'))
         return viewlist
 
