@@ -22,16 +22,12 @@ import copy
 
 class View(object):
     _node = None
-    _rootnode = None    
+    _db = None    
     _dbrow = None  # fast pytables writes
-    _verbose = False
-    _async = False
     
-    def __init__(self, node=None, rootnode=None, verbose=False, async=False):
+    def __init__(self, node=None, db=None):
         self._node = node
-        self._rootnode = rootnode
-        self._verbose = verbose
-        self._async = async
+        self._db = db
         
     def viewname(self):        
         """Name of current node which mirrors class name of corresponding view"""
@@ -56,8 +52,8 @@ class GroupView(View):
             if childnode._v_attrs.viewname == viewname:
                 thismodule = sys.modules[__name__]
                 viewclass = getattr(thismodule, childnode._v_attrs.viewclass)
-                return viewclass(childnode, self._rootnode)
-        raise IOError('undefined view ' + viewname)
+                return viewclass(childnode, self._db)
+        raise AttributeError, viewname
         
     def __iter__(self):
         raise NotImplementedError('FIXME: iterate over child nodes in group?')
@@ -71,17 +67,14 @@ class LeafView(View):
     _id_train = None
     _id_test = None
     _id_validate = None
-    _async = None
     _state = 'leaf'
     
-    def __init__(self, node=None, rootnode=None, verbose=False, async=False):
+    def __init__(self, node=None, db=None):
         self._node = node
-        self._rootnode = rootnode
-        self._async = async
-        self._verbose = verbose
+        self._db = db
         if self._node is not None:
-            self.set_split(strategy=None)
-            self.set_fold(0)
+            self._set_split(strategy=None)
+            self._set_fold(0)
         self._state = 'leaf'
             
     def __len__(self):
@@ -102,7 +95,7 @@ class LeafView(View):
 
     def __iter__(self):
         if self._state == 'kfold':
-            return imap(self.set_fold, iter(range(len(self._split))))
+            return imap(self._set_fold, iter(range(len(self._split))))
         elif self._state == 'train':
             return imap(self.read, iter(self._id_train))
         elif self._state == 'test':
@@ -142,34 +135,36 @@ class LeafView(View):
             return readlist
 
     def __call__(self, *args, **kwargs):
-        self.set_split(*args, **kwargs)
+        self._set_split(*args, **kwargs)
         return self
     
-    def set_split(self, strategy=None, split=None, folds=0, step=1, randomize=False, stratify=False):
+    def _set_split(self, strategy=None, split=None, folds=0, step=1, randomize=False, stratify=False):
+        """Internal function that sets a split without changing state"""
         if split is not None:
             self._split = split
-            self.set_fold(0)
+            self._set_fold(0)
             self._state = 'kfold'
         elif strategy == 'kfold':            
             self._split = viset.partition.kfold(len(self), folds, step, randomize=randomize, stratify=stratify)
-            self.set_fold(0)
+            self._set_fold(0)
             self._state = 'kfold'
         elif strategy in ['leave-one-out', 'loo', 'LOO']:
             self._split = viset.partition.leave_one_out(len(self), step, randomize=randomize, stratify=stratify)
-            self.set_fold(0)
+            self._set_fold(0)
             self._state = 'kfold'                    
         elif strategy in ['train-test']:
             self._split = viset.partition.train_test(len(self), step, randomize=randomize, stratify=stratify)            
-            self.set_fold(0)
+            self._set_fold(0)
             self._state = 'kfold'
         elif strategy in [None, 'all', 'leave_zero_out']:
             self._split = viset.partition.leave_zero_out(len(self), step, randomize=randomize, stratify=stratify)            
-            self.set_fold(0)            
+            self._set_fold(0)            
             self._state = 'leaf'
         else:
             IOError('unsupported splitting strategy ' + strategy)
 
-    def set_fold(self, k):
+    def _set_fold(self, k):
+        """Internal function that sets a fold index without changing state"""
         self._id_train = self._split[k][0]
         self._id_test = self._split[k][1]
         self._id_validate = self._split[k][2] 
@@ -177,7 +172,8 @@ class LeafView(View):
         return self
     
     def split(self, *args, **kwargs):
-        self.set_split(*args, **kwargs)
+        """Return a tuple containing splits"""
+        self._set_split(*args, **kwargs)
         if self._state == 'kfold' and len(self._split) > 1:
             return copy.copy(self.kfold)
         else:
@@ -228,8 +224,8 @@ class ImageView(LeafView):
     
     def read(self, id_img):
         imurl = self._node[id_img]['url']	
-        im = CachedImage(imurl, verbose=self._verbose)
-        if self._async:
+        im = CachedImage(imurl, cache=self._db._cache, verbose=self._db._verbose)
+        if self._db._async:
             return im  
         else:
             return im.get()
@@ -270,10 +266,10 @@ class CategorizationAnnotationView(LeafView):
 
     def read(self, id_anno):
         id_img = self._node[id_anno]['id_img']
-        imurl = self._rootnode.image[id_img]['url']
-        im = CachedImage(imurl, verbose=self._verbose)
+        imurl = self._db._obj.root.image[id_img]['url']
+        im = CachedImage(imurl, cache=self._db._cache, verbose=self._db._verbose)
         anno = self._node[id_anno]
-        if self._async:
+        if self._db._async:
             return (im,anno)  # numpy structured array tuple
         else:
             return (im.get(),anno)  # numpy structured array tuple
@@ -342,10 +338,10 @@ class DetectionAnnotationView(LeafView):
 
     def read(self, id_anno):
         id_img = self._node[id_anno]['id_img']
-        impath = self._rootnode.image[id_img]['url']
-        im = CachedImage(impath, verbose=self._verbose)
+        impath = self._db._obj.root.image[id_img]['url']
+        im = CachedImage(impath, cache=self._db._cache, verbose=self._db._verbose)
         anno = self._node[id_anno]
-        if self._async:
+        if self._db._async:
             return (im,anno)  # numpy structured array tuple
         else:
             return (im.get(),anno)  # numpy structured array tuple
@@ -391,15 +387,15 @@ class SegmentationAnnotationView(LeafView):
     
     def read(self, id_anno):
         id_img = self._node[id_anno]['id_img']
-        imurl = self._rootnode.image[id_img]['url']	
-        im = CachedImage(imurl, verbose=self._verbose)
+        imurl = self._db._obj.root.image[id_img]['url']	
+        im = CachedImage(imurl, cache=self._db._cache, verbose=self._db._verbose)
         maskurl = self._node[id_anno]['mask']
         if maskurl == '':
 			mask = None
         else:
-			mask = CachedImage(maskurl, verbose=self._verbose)
+			mask = CachedImage(maskurl, cache=self._db._cache, verbose=self._db._verbose)
         anno = self._node[id_anno]
-        if self._async:
+        if self._db._async:
             anno = {'id_category':anno['id_category'], 'category':anno['category'], 'mask':mask}            
             return (im,anno)  # numpy structured array tuple
         else:
@@ -408,4 +404,3 @@ class SegmentationAnnotationView(LeafView):
             anno = {'id_category':anno['id_category'], 'category':anno['category'], 'mask':mask}                        
             return (im.get(),anno)  # numpy structured array tuple
 
-
